@@ -151,3 +151,12 @@ is.directed <- function(graph) {
   .Call("R_igraph_is_directed", graph,
         PACKAGE="igraph")
 }
+
+get.edges <- function(graph, es) {
+  if (!is.igraph(graph)) {
+    stop("Not a graph object")
+  }
+  res <- .Call("R_igraph_edges", graph, as.igraph.es(es),
+               PACKAGE="igraph")
+  matrix(res, nc=2, byrow=TRUE)
+}
