@@ -103,6 +103,12 @@ int igraph_dqueue_push    (igraph_dqueue_t* q, igraph_real_t elem);
 #include "igraph_pmt_off.h"
 #undef BASE_CHAR
 
+#define BASE_BOOL
+#include "igraph_pmt.h"
+#include "vector.h"
+#include "igraph_pmt_off.h"
+#undef BASE_BOOL
+
 int igraph_vector_order2(igraph_vector_t *v);
 
 /* -------------------------------------------------- */
